import opendreams.proxy.OpenDreamsException;
import opendreams.proxy.OpenDreamsProxy;
import opendreams.proxy.Project;
import tecgraf.openbus.DRMAA.AlreadyActiveSessionException;
import tecgraf.openbus.DRMAA.AuthorizationException;
import tecgraf.openbus.DRMAA.DefaultContactStringException;
import tecgraf.openbus.DRMAA.DeniedByDrmException;
import tecgraf.openbus.DRMAA.DrmCommunicationException;
import tecgraf.openbus.DRMAA.DrmsExitException;
import tecgraf.openbus.DRMAA.DrmsInitException;
import tecgraf.openbus.DRMAA.ExitTimeoutException;
import tecgraf.openbus.DRMAA.InternalException;
import tecgraf.openbus.DRMAA.InvalidArgumentException;
import tecgraf.openbus.DRMAA.InvalidContactStringException;
import tecgraf.openbus.DRMAA.InvalidJobException;
import tecgraf.openbus.DRMAA.InvalidJobTemplateException;
import tecgraf.openbus.DRMAA.JobInfo;
import tecgraf.openbus.DRMAA.NoActiveSessionException;
import tecgraf.openbus.DRMAA.NoDefaultContactStringSelectedException;
import tecgraf.openbus.DRMAA.OutOfMemoryException;
import tecgraf.openbus.DRMAA.Session;
import tecgraf.openbus.DRMAA.TryLaterException;
import tecgraf.openbus.opendreams.IOpenDreams;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplate;

/**
 * Teste de acesso ao servio OpenDreams do OpenBus.
 * 
 * @author Tecgraf PUC-Rio
 * 
 */
public class SimpleDemoOpenDreams {
  /**
   * Executa o teste.
   * 
   * @param args argumentos da linha de comando. No possui argumentos de
   *        entrada.
   * @throws Exception se ocorrer alguma falha na execuo do teste
   */
  public static void main(String[] args) throws Exception {
    new SimpleDemoOpenDreams().run();
  }

  private void run() {
    try {
      OpenDreamsProxy opendreamsProxy = new OpenDreamsProxy();
      opendreamsProxy.init();
      System.out.println("** Recuperando o componente OpenDreams...");
      IOpenDreams opendreams = opendreamsProxy.getIOpenDreams();
      Session session = opendreams.getSession(opendreamsProxy.getProjectName());
      session.init("");
      OpenDreamsJobTemplate jt =
        (OpenDreamsJobTemplate) session.createJobTemplate();
      jt.setRemoteCommand("execAlgo");
      jt.setArgs(new String[] { "-name", "teste", "-version", "1.0.0" });
      jt.setJobDescription("Teste da Julia");
      jt.addJobParameter("qtde", "6");
      jt.addJobParameter("duracao", "3");
      jt.setOutputPath("saida.out");
      jt.setEmail(new String[] { "mjulia@tecgraf.puc-rio.br" });
      System.out.println("** Solicitando a execucao de "
        + jt.getRemoteCommand());
      System.out.print("** Argumentos: ");
      for (String arg : jt.getArgs()) {
        System.out.print(arg);
        System.out.print(" ");
      }
      System.out.println();
      System.out.println("** Parametro qtde: " + jt.getJobParameter("qtde"));
      System.out.println("** Parametro duracao: "
        + jt.getJobParameter("duracao"));
      System.out.println("** OutputPath: " + jt.getOutputPath());
      String jobName = session.runJob(jt);
      System.out.println("** Id do comando: " + jobName);
      session.deleteJobTemplate(jt);

      JobInfo jobInfo = session._wait(jobName, Session.TIMEOUT_WAIT_FOREVER);
      System.out.println("** Resultado: hasExited = " + jobInfo.hasExited
        + ": exitStatus = " + jobInfo.exitStatus);

      Project project = opendreamsProxy.getProject();
      byte[] buffer = project.getDataFrom("saida.out");
      System.out.println("** O log de sada do comando :");
      System.out.println(new String(buffer));
      session.exit();
      opendreamsProxy.close();
    }
    catch (OpenDreamsException e) {
      e.printStackTrace();
    }
    catch (AuthorizationException e) {
      handleException(e.message, e);
    }
    catch (InternalException e) {
      handleException(e.message, e);
    }
    catch (InvalidContactStringException e) {
      handleException(e.message, e);
    }
    catch (NoDefaultContactStringSelectedException e) {
      handleException(e.message, e);
    }
    catch (DefaultContactStringException e) {
      handleException(e.message, e);
    }
    catch (DrmCommunicationException e) {
      handleException(e.message, e);
    }
    catch (DrmsInitException e) {
      handleException(e.message, e);
    }
    catch (AlreadyActiveSessionException e) {
      handleException(e.message, e);
    }
    catch (OutOfMemoryException e) {
      handleException(e.message, e);
    }
    catch (InvalidArgumentException e) {
      handleException(e.message, e);
    }
    catch (TryLaterException e) {
      handleException(e.message, e);
    }
    catch (NoActiveSessionException e) {
      handleException(e.message, e);
    }
    catch (DeniedByDrmException e) {
      handleException(e.message, e);
    }
    catch (InvalidJobTemplateException e) {
      handleException(e.message, e);
    }
    catch (InvalidJobException e) {
      handleException(e.message, e);
    }
    catch (ExitTimeoutException e) {
      handleException(e.message, e);
    }
    catch (DrmsExitException e) {
      handleException(e.message, e);
    }
  }
  
  private void handleException (String message, Exception e) {
    e.initCause(new Exception(message));
    e.printStackTrace();
  }
}
