import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import opendreams.proxy.OpenDreamsProxy;
import tecgraf.openbus.DRMAA.Session;
import tecgraf.openbus.opendreams.IOpenDreams;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplate;

/**
 * Teste de robustez do acesso ao servio OpenDreams do OpenBus.
 * 
 * @author Tecgraf PUC-Rio
 * 
 */
public class ThreadedDemoOpenDreams {
  private static final String USERNAME_BASE = "usr";
  private static final String PROJECT_NAME = "prj";
  private static final int NUM_TASKS = 10;
  private static final int NUM_TESTS_PER_TASK = 5;
  private static final int NUM_THREADS = 5;
  private static final int NUM_USERS = 1;

  private static OpenDreamsProxy opendreamsProxy;
  private static IOpenDreams opendreams;
  private static int num_errors;

  private static class Task implements Callable<Void> {
    private static Map<Session, Integer> sessions =
      new HashMap<Session, Integer>();

    private String id;
    private String user;

    private Task(String id, String user) {
      this.id = id;
      this.user = user;
    }

    public static synchronized void initSession(Session session)
      throws Exception {
      Integer n = sessions.get(session);
      if (n == null) {
        n = 0;
        session.init("");
      }
      n++;
      sessions.put(session, n);
    }

    public static synchronized void exitSession(Session session)
      throws Exception {
      Integer n = sessions.get(session);
      n--;
      if (n == 0) {
        session.exit();
        sessions.remove(session);
      }
      else {
        sessions.put(session, n);
      }
    }

    public Void call() {
System.out.println("Iniciando task ["+id+"/"+user+"]");
      try {
        opendreamsProxy.setThreadDelegate(user);
System.out.println("["+id+"/"+user+"] Criando sesso");
        Session session = opendreams.getSession(PROJECT_NAME);
System.out.println("["+id+"/"+user+"] Iniciando sesso");
        initSession(session);
        for (int i=1; i<=NUM_TESTS_PER_TASK; i++) {
          String input = id+"_"+i;
          String outfilename = "echo_"+input+".out";
System.out.println("["+id+"/"+user+"] entrada = "+input);
System.out.println("["+id+"/"+user+"] sada = "+outfilename);
System.out.println("["+id+"/"+user+"] Criando template");
          OpenDreamsJobTemplate jt =
            (OpenDreamsJobTemplate)session.createJobTemplate();
System.out.println("["+id+"/"+user+"] Registrando execAlgo");
          jt.setRemoteCommand("execAlgo");
System.out.println("["+id+"/"+user+"] Ajustando argumentos");
          jt.setArgs(new String[] { "-name", "echo", "-version", "1.0.0" });
System.out.println("["+id+"/"+user+"] Ajustando descrio");
          jt.setJobDescription(input);
System.out.println("["+id+"/"+user+"] Ajustando parmetros");
          jt.addJobParameter("p", input);
System.out.println("["+id+"/"+user+"] Ajustando caminho de sada");
          jt.setOutputPath(outfilename);
System.out.println("["+id+"/"+user+"] Executando comando");
          String job = session.runJob(jt);
System.out.println("["+id+"/"+user+"] Removendo template");
          session.deleteJobTemplate(jt);
System.out.println("["+id+"/"+user+"] Aguardando comando");
          session._wait(job, Session.TIMEOUT_WAIT_FOREVER);
System.out.println("["+id+"/"+user+"] Obtendo projeto");
/*
          Project project = opendreamsProxy.getProject(PROJECT_NAME);
System.out.println("["+id+"/"+user+"] Lendo sada");
          String output = new String(project.getDataFrom(outfilename));
System.out.println("["+id+"/"+user+"] Verificando resultado");
          output = output.trim();
          if (!input.equals(output)) {
            handleError(input, output);
          }
*/
        }
System.out.println("["+id+"/"+user+"] Terminando sesso");
        exitSession(session);
      }
      catch (Throwable t) {
        handleError(t);
      }
System.out.println("Terminando task "+id+"/"+user);
      return null;
    }
  }

  /**
   * Executa o teste.
   * 
   * @param args argumentos da linha de comando. No possui argumentos de
   *        entrada.
   */
  public static void main(String[] args) {
    try {
      System.out.println("Iniciando teste. Parmetros:");
      System.out.println("- Nmero de tasks: "+NUM_TASKS);
      System.out.println("- Nmero de testes por task: "+NUM_TESTS_PER_TASK);
      System.out.println("- Nmero de threads: "+NUM_THREADS);
      System.out.println("- Nmero de usurios: "+NUM_USERS);
      opendreamsProxy = new OpenDreamsProxy();
      opendreamsProxy.init();
      opendreams = opendreamsProxy.getIOpenDreams();
      num_errors = 0;
      runTasks();
      opendreamsProxy.close();
      // Fora a sada para no ter que esperar o encerramento do thread pool.
      if (num_errors == 0) {
        System.out.println("Teste terminado. Nenhum erro encontrado.");
        System.exit(0);
      }
      else {
        System.out.println("Total de erros encontrados: "+num_errors+".");
        System.exit(1);
      }
    }
    catch (Throwable t) {
      handleError(t);
    }
  }

  private static void runTasks() throws Exception {
    ExecutorService executor = Executors.newFixedThreadPool(NUM_THREADS);
    Collection<Callable<Void>> tasks = new ArrayList<Callable<Void>>(NUM_TASKS);
    for (int i=1; i<=NUM_TASKS; i++) {
      String id = "Task_"+i;
      String user = USERNAME_BASE+((i%NUM_USERS)+1);
      tasks.add(new Task(id, user));
    }
    List<Future<Void>> futures = executor.invokeAll(tasks);
    for (Future<Void> f : futures) {
      f.get();
    }
    executor.shutdown();
  }

  private static synchronized void handleError(String input, String output) {
    System.out.println("Erro: |"+input+"| != |"+output+"|");
    num_errors++;
  }

  private static synchronized void handleError(Throwable t) {
    t.printStackTrace();
    System.exit(1);
  }
}
